/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 Module:       FGInertial.cpp
 Author:       Jon S. Berndt
 Date started: 09/13/00
 Purpose:      Encapsulates the inertial frame forces (coriolis and centrifugal)

 ------------- Copyright (C) 2000  Jon S. Berndt (jsb@hal-pc.org) -------------

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA  02111-1307, USA.

 Further information about the GNU General Public License can also be found on
 the world wide web at http://www.gnu.org.

FUNCTIONAL DESCRIPTION
--------------------------------------------------------------------------------

HISTORY
--------------------------------------------------------------------------------
09/13/00   JSB   Created

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
INCLUDES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#include "FGInertial.h"
#include "FGPosition.h"
#include "FGMassBalance.h"

static const char *IdSrc = "$Id: FGInertial.cpp,v 1.8 2001/06/20 20:29:32 curt Exp $";
static const char *IdHdr = ID_INERTIAL;

extern short debug_lvl;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CLASS IMPLEMENTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


FGInertial::FGInertial(FGFDMExec* fgex) : FGModel(fgex),
    vForces(3),
    vOmegaLocal(3),
    vRadius(3)
{
  Name = "FGInertial";

  vRadius.InitMatrix();

  if (debug_lvl & 2) cout << "Instantiated: FGInertial" << endl;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FGInertial::~FGInertial(void)
{
  if (debug_lvl & 2) cout << "Destroyed:    FGInertial" << endl;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bool FGInertial::Run(void)
{
  float stht, ctht, sphi, cphi;

  if (!FGModel::Run()) {

    stht = sin(Rotation->GetEuler(eTht));
    ctht = cos(Rotation->GetEuler(eTht));
    sphi = sin(Rotation->GetEuler(ePhi));
    cphi = cos(Rotation->GetEuler(ePhi));

    vForces(eX) = -GRAVITY*stht;
    vForces(eY) =  GRAVITY*sphi*ctht;
    vForces(eZ) =  GRAVITY*cphi*ctht;
    
    // The following equation for vOmegaLocal terms shows the angular velocity
    // calculation _for_the_local_frame_ given the earth's rotation (first set)
    // at the current latitude, and also the component due to the aircraft
    // motion over the curved surface of the earth (second set).

    vOmegaLocal(eX) = OMEGA_EARTH * cos(Position->GetLatitude());
    vOmegaLocal(eY) = 0.0;
    vOmegaLocal(eZ) = OMEGA_EARTH * -sin(Position->GetLatitude());

    vOmegaLocal(eX) +=  Position->GetVe() / Position->GetRadius();
    vOmegaLocal(eY) += -Position->GetVn() / Position->GetRadius();
    vOmegaLocal(eZ) +=  0.00;

//    vForces = State->GetTl2b()*(-2.0*vOmegaLocal * Position->GetVel());

    vRadius(3) = Position->GetRadius();
    vForces += State->GetTl2b()*(vOmegaLocal * (vOmegaLocal * vRadius));

    vForces *= MassBalance->GetMass(); // IMPORTANT !!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    return false;
  } else {
    return true;
  }
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bool FGInertial::LoadInertial(FGConfigFile* AC_cfg)
{
  return true;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void FGInertial::Debug(void)
{
    //TODO: Add your source code here
}

